/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.nodes;

import java.util.logging.Logger;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.starbeans.modules.bistro.services.BioService;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.WeakListeners;

/**
 *
 * @author George Waldon
 */
public class ServiceNode extends AbstractNode implements ChangeListener {

    private static final Logger LOGGER = Logger.getLogger(ServiceNode.class.getName());
    
    private BioService service;
    
    public ServiceNode(BioService service, Children ch) {
        super(ch);
        if(service!=null) //Folder
            setService(service);
    }

    @Override
    public boolean canRename() {
        return false;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public boolean canCopy() {
        return false;
    }

    @Override
    public boolean canDestroy() {
        return false;
    }

    public void stateChanged(ChangeEvent e) {
        processService();
    }

    protected void processService() {
        setIconBaseWithExtension(service.getIconBase());
        setDisplayName(service.getName());
        setName(service.getID());
    }

    protected void setService(BioService service) {
        this.service = service;
        service.addChangeListener(WeakListeners.create(
                ChangeListener.class, this, service)); 
        stateChanged(new ChangeEvent(service));
    }
}
