/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.nodes;

import java.util.LinkedList;
import java.util.logging.Logger;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * @author George Waldon
 */
class ServiceNodeChildren <T> extends Children.Keys <DataObject> {

    private static final Logger LOGGER = Logger.getLogger(ServiceNodeChildren.class.getName());
    
    private DataObject dto;
    private boolean initialized;
    
    public ServiceNodeChildren(DataObject dto) {
        this.dto = dto;
    }
    
    protected void addNotify() {
        refreshKeys ();
        initialized = true;        
    }
    
    protected void removeNotify() {
        setKeys(java.util.Collections.<DataObject>emptySet());
        initialized = false;
    }
    
    void refreshKeys() {
        final LinkedList<DataObject> keys = new LinkedList<DataObject>();
        if(dto instanceof DataFolder) {
            DataFolder dtf = (DataFolder) dto;
            DataObject[] objs = dtf.getChildren();
            for(int i=0;i<objs.length;i++)
                keys.add(objs[i]);
        } else
            keys.add(dto);
        setKeys(keys);
    }
    
    @Override
    protected Node[] createNodes(DataObject key) {
        Node n = null;
        if (key instanceof DataFolder) {
            n = ServiceNodeFactory.DEFAULT_NODE_FACTORY.createServiceNodeFolder((DataFolder)key);
        } else
            n = ServiceNodeFactory.DEFAULT_NODE_FACTORY.createServiceNode(key);
        if(n!=null)
            return new Node[]{n};
        LOGGER.info("A bioservice node is missing");
        return null;
    }

}
