/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.nodes;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.starbeans.modules.bistro.services.BioService;
import org.openide.cookies.InstanceCookie;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 *
 * @author George Waldon
 */
public class ServiceNodeFactory  {
    
    /** Default instance of the factory. */
    public static final ServiceNodeFactory DEFAULT_NODE_FACTORY = new ServiceNodeFactory();

    private static final Logger LOGGER = Logger.getLogger(
                ServiceNodeFactory.class.getName());
    

    public Node createServiceNode (DataObject obj) {
        InstanceCookie cookie = obj.getCookie(InstanceCookie.class);
        if (cookie instanceof InstanceCookie.Of) {
            if (((InstanceCookie.Of) cookie).instanceOf(BioService.class)) {
                try {
                    BioService bs = (BioService) cookie.instanceCreate();
                    return new ServiceNode(bs, Children.LEAF);
                } catch (IOException ex) {
                    LOGGER.log(Level.INFO, "", ex);
                } catch (ClassNotFoundException ex) {
                    LOGGER.log(Level.INFO, "", ex);
                }
            }
        }
        LOGGER.info("A bioservice node is missing");
        return null;
    }
    
    public Node createServiceNodeFolder(DataFolder dataFolder) {
        return new ServiceNodeFolder(dataFolder,new ServiceNodeChildren(dataFolder));
    }
    
    public Node createErrorNode() {
        AbstractNode nod = new AbstractNode(Children.LEAF);
        nod.setName(NbBundle.getBundle(ServiceNodeFactory.class).getString("NOD_Error"));
        nod.setIconBaseWithExtension("org/starbeans/modules/bistro/resources/error.gif");
        return nod;
    }    
    
    /** 
     * @return a Node indicating a waiting state.
    */
    public Node createWaitNode () {
        AbstractNode nod = new AbstractNode(Children.LEAF);
        nod.setName(NbBundle.getBundle(ServiceNodeFactory.class).getString("NOD_Wait"));
        nod.setIconBaseWithExtension("org/starbeans/modules/bistro/resources/wait.gif");
        return nod;
    }

    
}
