/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.onlineservices;

import java.util.ArrayList;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.starbeans.modules.bistro.services.BioService;
import org.starbeans.modules.bistro.services.BioServiceRegistry;
import org.starbeans.modules.bistro.services.DuplicateProcessorNameException;
import org.starbeans.modules.bistro.services.ProcessorCreationException;
import org.starbeans.modules.bistro.services.ServiceProcessor;
import org.starbeans.modules.bistro.services.ServiceProcessorFactory;
import org.starbeans.modules.bistro.services.Worker;
import org.openide.util.Lookup;

/**
 *
 * @author George Waldon
 */
public abstract class OnlineService implements BioService, ServiceProcessor {

    private final ArrayList<ChangeListener> listeners =
            new ArrayList<ChangeListener>();
    
    private ServiceProcessorFactory factory;
    
    private String description = "";
    private String name = "";
    
    public OnlineService() {
        BioServiceRegistry registry = Lookup.getDefault().lookup(BioServiceRegistry.class);
        registry.register(this);
    }
    
    public void setName(String n) {
        if(n==null)
            throw new IllegalArgumentException("Name was null");
        name = n;
    }
    public String getName() {
        return name;
    }

    public void setDescription(String d) {
        description = d;
    }
    
    public String getDescription() {
        return description;
    }

    /** Implementations of this service must provide instances of the worker.
     */
    public abstract Worker getWorker();

    public void addChangeListener(ChangeListener listener) {
        listeners.add(listener);
    }
    
    public void removeChangeListener(ChangeListener listener) {
        listeners.remove(listener);
    }
    
    protected void notifyChange() {
        for ( ChangeListener listener : listeners ) {
            listener.stateChanged(new ChangeEvent(this));
        }
    }

    public String getIconBase() {
        return "org/starbeans/modules/bistro/resources/local.png";
    }

    public void setProcessorfactory(ServiceProcessorFactory factory) {
        this.factory = factory;
    }

    
    public ServiceProcessorFactory getProcessorFactory() {
        if(factory==null)
            factory = new OnlineProcessorFactory();
        return factory;
    }
    
    /* everything local */
    private class OnlineProcessorFactory implements ServiceProcessorFactory {

        public ServiceProcessor createProcessor(String name) throws ProcessorCreationException, DuplicateProcessorNameException {
            //assert(name!=null && name.equals(this.getClass().getName()));
            return new OnlineProcessor(OnlineService.this);
        }
        
    }
    
    private static class OnlineProcessor implements ServiceProcessor {
        
        OnlineService ser;
        
        private OnlineProcessor(OnlineService ser) {
            this.ser = ser;
        }

        public Worker getWorker() {
            return ser.getWorker();
        }
        
    }

}
