/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.services;

import org.openide.util.Lookup;

/** A registry for all bioservices.
 *
 * @author George Waldon
 */

public interface BioServiceRegistry {
    
    /** Provide a lookup dedicated to varied object in Bistro.
     */
    public Lookup getBistroLookup();
    
    /* Find service by class name.
     * 
     * param name the name of the class of the service to look for
     * return the coresponding service.
     */
    public BioService findService (String className);
    
    /* Find service by id.
     * 
     */
    public BioService findServicebyID (String id);
    
    /** Register a service by conversion in the registry lookup.
     */
    public void register (BioService service);
    
    /** Unregisters the service.
     */
    public void unregister (BioService service);

}
