/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.starbeans.modules.bistro.services;

import java.util.logging.Logger;
import javax.swing.event.ChangeListener;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 *
 * @author George Waldon
 */
public class BioServiceRoot implements BioService {
    
    private static final Logger LOGGER = Logger.getLogger(BioServiceRoot.class.getName());
    
    private static BioService rootservice;
    
    public static BioService getDefault() {
        if(rootservice==null)
            rootservice = new BioServiceRoot();
        return rootservice;
    }
    
    private BioServiceRoot() {
        BioServiceRegistry registry = Lookup.getDefault().lookup(BioServiceRegistry.class);
        registry.register(this);
    }
    
    public String getID() {
        return "root"; //NOI18N
    }

    public String getName() { 
        return NbBundle.getMessage(BioServiceRoot.class, "LBL_root");
    }

    public String getDescription() {
        throw new UnsupportedOperationException("Not supported yet."); //NOI18N
    }

    public Worker getWorker() {
        throw new UnsupportedOperationException("Nothing to do here."); //NOI18N
    }

    public void addChangeListener(ChangeListener listener) {
        //do nothing
    }

    public void removeChangeListener(ChangeListener listener) {
        //do nothing
    }

    public String getIconBase() {
        return NbBundle.getMessage(BioServiceRoot.class, "ICO_root");
    }

    public ServiceProcessorFactory getProcessorFactory() {
        LOGGER.info("nothing to get here"); //NOI18N
        assert false;
        return null;
    }

    public void setProcessorfactory(ServiceProcessorFactory factory) {
        LOGGER.info("nothing to set here"); //NOI18N
        assert false;
    }

}
