/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.services;

import java.util.Map;
import org.starbeans.modules.bistro.task.TaskExecutionException;

/** Marker interface for workers, those who are actually getting
 *  the job done.
 *
 * @author George Waldon
 */
public interface Worker {
    /**
	 * Given a Map of DataThing objects as input, execute the work and 
         * return a map of DataThing objects as outputs, with the port
	 * names acting as keys in the map in both cases.
	 * 
	 * @exception TaskExecutionException
	 *   thrown if there is an error during invocation of the task.
	 */
	public Map execute(Map inputMap) throws TaskExecutionException;

	/**
	 * Get an array of the names of input ports for this processor
	 */

	public String[] inputNames();

	/**
	 * Get an array of the string types for the inputs defined by the
	 * inputNames() method, may use any valid data type specifier.
	 */
	public String[] inputTypes();

	/**
	 * Names of the output ports
	 */
	public String[] outputNames();

	/**
	 * Types of the output ports
	 */
	public String[] outputTypes();


}
