/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/

package org.starbeans.modules.bistro.task;

import java.util.Map;
import org.starbeans.modules.bistro.services.ServiceProcessor;

/**
 *
 * @author George Waldon
 */
public class ProcessorTaskFactory {

    /** No instantiation.
     *
     */
    private ProcessorTaskFactory() {
        
    }
    
    /** Bistro way of getting the TaskWorker.
     * 
     * @param p a ServiceProcessor
     * @return the ProcesorTaskWorker
     */
    public static ProcessorTaskWorker createTaskWorker(ServiceProcessor p) {
        return new Task(p);
    }
    
    private static class Task implements ProcessorTaskWorker {
        
        private ServiceProcessor sprocessor;

        Task(ServiceProcessor p) {
            this.sprocessor = p;
        }
        
        public Map execute(Map inputMap) throws TaskExecutionException {
            return sprocessor.getWorker().execute(inputMap);
        }
        
    }
}
