/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Portions Copyrighted 2008-2010 George Waldon
 */

package org.starbeans.modules.corelite.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Iterator;
import java.util.Set;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.openide.ErrorManager;
import org.openide.awt.Actions;
import org.openide.awt.DynamicMenuContent;
import org.openide.explorer.view.MenuView;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.loaders.TemplateWizard;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.starbeans.modules.corelite.utilities.UIUtilities;

/** An action based on a single template.
 *
 * @author George Waldon
 */
public abstract class TemplateAction extends NewTemplateInlineAction {
    
    protected abstract String getTemplatePath();
    
    @Override
    protected JMenuItem getPopupPresenter(final Lookup actionContext, Action action) {
        FileObject fo = FileUtil.getConfigFile(getTemplatePath());
        DataObject obj;
        try {
            obj = DataObject.find(fo);
        } catch (DataObjectNotFoundException ex) {
            Exception e = new Exception("DataObject not found; check template path"); //NOI18N
            e.initCause(ex);
            ErrorManager em = ErrorManager.getDefault();
            em.notify(ErrorManager.WARNING, e);
            return createEmptyMenuItem();
        }
        
        Node[] nodes = new Node[0];
        //if (actionContext != null) {
            nodes = getNodesFromLookup (actionContext);
        //}
        final Node n = (nodes.length == 1) ? nodes[0] : null;
        if(n==null)
            return createEmptyMenuItem();
        if(isValidTemplate(obj)) {
            return new Item(n,obj, this.isEnabled ());
        } else if(fo.isFolder()) {
            JMenuItem menu = new InlineTemplate(actionContext);
            Actions.connect(menu, action, true);
            return menu;
        } else  {
            return createEmptyMenuItem();
        }
    }

    private boolean isValidTemplate (DataObject template) {
        return (template != null) && template.isTemplate () && template.isValid ();
    }
    
    private class InlineTemplate extends JMenuItem implements DynamicMenuContent {
        
        Lookup actionContext;
        
        InlineTemplate(Lookup actionContext) {
            super();
            this.actionContext = actionContext;
        }
        
        public JComponent[] getMenuPresenters() {
            return synchMenuPresenters(new JComponent[0]);
        }
        
        public JComponent[] synchMenuPresenters(JComponent[] items) {
            final Node fnode = getNodeTemplate(getTemplatePath());
            if(fnode == null)
                throw new IllegalStateException("Node is null"); //NOI18N
            JMenuItem menu = new MenuView.Menu(null, new NewTemplateInlineAction.TemplateActionListener(actionContext), false) {
                // this is the only place MenuView.Menu needs the node ready
                // so lets prepare it on-time
                @Override
                public JPopupMenu getPopupMenu() {
                    if (node == null) node = fnode;
                    return super.getPopupMenu();
                }
            };
            Icon icon = new ImageIcon(fnode.getIcon(java.beans.BeanInfo.ICON_COLOR_16x16));
            if(icon!=null)
                menu.setIcon(icon);
            menu.setText(fnode.getDisplayName());
            return new JComponent[] {menu};
        }
    }
    
    /** Getter for the node corresponding to the template.
     */
    private static Node getNodeTemplate(String path) {
        FileObject fo = FileUtil.getConfigFile(path); // NOI18N
        if (fo != null) {
            try {
                return DataObject.find(fo).getNodeDelegate();
            } catch (DataObjectNotFoundException ex) {
                //shoudn't happen;
            }
        }
        return null;
    }
    
    /** Helper method. Creates empty menu item. */
    private static JMenuItem createEmptyMenuItem() {
        JMenuItem empty = new JMenuItem(NbBundle.getMessage(MenuView.class, "EmptySubMenu"));
        empty.setEnabled(false);
        return empty;
    }

    private class Item extends JMenuItem implements HelpCtx.Provider, ActionListener {
        Node n;
        DataObject template; // Null means no template -> show the chooser
        
        public Item(Node n, DataObject template, boolean writable) {
            super();
            this.n = n;
            this.template = template;
            
            setText(template == null ?
                NbBundle.getMessage(DataObject.class, "NewTemplateAction") :
                template.getNodeDelegate().getDisplayName()
                );
            
            if (template == null) {
                setIcon(TemplateAction.this.getIcon());
            } else {
                setIcon(new ImageIcon(template.getNodeDelegate().getIcon(java.beans.BeanInfo.ICON_COLOR_16x16)));
            }
            
            addActionListener(this);
            // recommendation from issue 32191, don't enable popup menu items on read-only folders
            setEnabled(writable);
        }
        
        /** Get context help for this item.*/
        public HelpCtx getHelpCtx() {
            if (template != null) {
                return template.getHelpCtx();
            }
            return TemplateAction.this.getHelpCtx();
        }
        
        /** Invoked when an action occurs. */
        public void actionPerformed(ActionEvent e) {
            doShowWizard(template, n);
        }
    }
    
    private void doShowWizard(DataObject template, Node node) {
        DataFolder targetFolder = null;
        Node folder = node;
        while (targetFolder == null && folder != null) {
            targetFolder = folder == null ? null : folder.getCookie (DataFolder.class);
            folder = folder.getParentNode ();
        }
        TemplateWizard wizard = getWizard (node);
        try {
            wizard.setTargetName(null);
            //STARBEANS: open new file
             Set<DataObject> resultSet = wizard.instantiate(template, targetFolder);
             if (resultSet == null || resultSet.isEmpty()) {
                // no new object, no work
                return;
            }

            Iterator<DataObject> it = resultSet.iterator();
            while (it.hasNext()) {
                DataObject newDO = it.next();
                if (newDO != null) {
                    UIUtilities.openAndSelectNewObject(newDO);
                    break;
                }
            }

        } catch (IOException e) {
            ErrorManager em = ErrorManager.getDefault();
            em.annotate(e, NbBundle.getMessage(DataObject.class, "EXC_TemplateFailed"));
            em.notify(e);
        }
    }
}
