/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.WeakHashMap;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Segment;
import javax.swing.text.StyledDocument;
import org.openide.cookies.EditorCookie;
import org.openide.loaders.DataObject;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Node;
import org.openide.text.CloneableEditorSupport;
import org.openide.text.PositionBounds;
import org.openide.text.PositionRef;
import org.openide.util.RequestProcessor;
import org.openide.util.WeakSet;
import org.starbeans.modules.seqformat.model.Binding;
import org.starbeans.modules.seqformat.model.BindingFactory;
import org.starbeans.modules.seqformat.model.DefaultSeqModel;
import org.starbeans.modules.seqformat.model.ElementImpl;
import org.starbeans.modules.seqformat.model.ElementOrder;
import org.starbeans.modules.seqformat.model.SeqModel;
import org.starbeans.modules.seqformat.model.WrapperFactory;
import org.starbeans.modules.seqformat.parser.ParsingSupport;
import org.starbeans.modules.seqformat.parser.SeqParser;
import org.starbeans.modules.seqformat.source.Element;
import org.starbeans.modules.seqformat.source.ElementProperties;
import org.starbeans.modules.seqformat.source.SeqElement;
import org.starbeans.modules.seqformat.source.SrcCookie;
import org.starbeans.modules.seqformat.source.SrcElement;
import org.starbeans.modules.seqformat.srcgen.DocBinding;
import org.starbeans.modules.seqformat.srcgen.SrcText;
import org.starbeans.modules.seqformat.srcgen.TextBinding;

/** A parser environment to communicate between the DataObject and the SeqParser
 *
 * @author  sdedic, George Waldon
 */
public abstract class ParserEnv implements DocBinding.Env, SrcCookie.Env, SeqParser.Env, SeqModel.Env,
PropertyChangeListener, DocumentListener {

    protected SeqDataObject         sdo;
    protected ParsingSupport        seqparser;
    protected DocBinding            docBinding;
    protected WrapperFactory        wrapper;
    private boolean                 documentLoaded;
    private CloneableEditorSupport  cloneableEditSupp;
    private int                     suspended;
    private boolean 		    disableCookieEvents;
    private boolean                 editorBound;
    private boolean                 initialized = false;
    
    private Reference<StyledDocument> DocRef = null;
    private Set<SeqElement>         knownSeqs;
        
    public ParserEnv(SeqDataObject dataobject) {
        this.sdo = dataobject;
        knownSeqs = new WeakSet<SeqElement>();
    }
    
    /** Initialize from subclass; wrapper is set in
     * subclasses.
     */
    protected void Initialize(DefaultSeqModel model) {
        if (wrapper==null) {
            throw new InternalError("wrapper is null!");
        } else
            initialized = true;
        
        if (docBinding == null)
            this.docBinding = new SrcText(this);                
        this.seqparser = new ParsingSupport(this, sdo, docBinding, model, model);
        
        /* Add listener to SrcElement */
        seqparser.getSource().addPropertyChangeListener(this);
        /* Add listener to DataObject */
        sdo.addPropertyChangeListener(this);
        /* Add listener to SeqELement */
        if (seqparser.getStatus() != SrcElement.STATUS_NOT)    
            rescanSource(seqparser.getSource());
    }

    
    /**
     * Notifies the mediator that a CloneablEditorSupport has been created.
     */
    public void cloneableSupportCreated(CloneableEditorSupport edsupp) {
        bindEditorSupport(edsupp);
    }
    
    public SeqParser getParser() {
        return seqparser;
    }
    
    private CloneableEditorSupport getEditor() {
        return sdo.getEditor();
        //return (CloneableEditorSupport)(sdo).getCookie(CloneableEditorSupport.class);  //Not a cookie!
    }

    /* Remains of cookie storage on Elements. No current usage. */
//    protected <T extends Node.Cookie> T lookupCookie(Element el, Class<T> clazz) {
//        synchronized (cookieMap) {
//            Object o = cookieMap.get(el);
//            if (o == null)
//                return null;  
//            if (o instanceof CookieSet) 
//                return ((CookieSet)o).getCookie(clazz);            
//            if (o.getClass().isAssignableFrom(clazz))
//                return (T) o;
//            return null;
//        }
//    }    
    
//================== Pseudo implementation of SrcCookie ======================//
 
    public SrcElement getSource() {
        return seqparser.getSource();
    }
    
//========================= SrcCookie.Env ===================================//
    
    private volatile Map<Element,WeakReference<javax.swing.text.Element>>  swingElementMap;

    public StyledDocument getDocument() {
        return findEditorSupport().getDocument();
    }
        
    /* note return of SrcElement on special boundaries */
    public Element findElement(int offset) {
        javax.swing.text.Element swingEl = sourceToText(getSource());
        
        while (swingEl != null) {
            if (swingEl.isLeaf()) {
                return ((TextElement)swingEl).getSourceElement();
            }
            int elIndex = swingEl.getElementIndex(offset); 
            if (elIndex == -1)
                return ((TextElement)swingEl).getSourceElement();
            swingEl = swingEl.getElement(elIndex);
        }
        return null;
    }    

    public Element textToSource(javax.swing.text.Element element) throws NoSuchElementException {
        if (!(element instanceof TextElement)) {
            throw new NoSuchElementException();
        }
        TextElement el = (TextElement)element;
        return el.getSourceElement();
    }

    /**
     * Returns null, if the underlying document has not yet been loaded
     */
    public javax.swing.text.Element sourceToText(Element element) {
        javax.swing.text.Document d;
        try {
            d = findEditorSupport().openDocument();
        } catch (IOException ex) {
            IllegalStateException x = new IllegalStateException("Could not load document");
            x.initCause(ex);
            throw x;
        }
        if (swingElementMap == null) {
            synchronized (this) {
                if (swingElementMap == null)
                    swingElementMap = new WeakHashMap<Element,WeakReference<javax.swing.text.Element>>(75);
            }
        }
        Reference r = swingElementMap.get(element);
        javax.swing.text.Element e = r == null ? null : (javax.swing.text.Element)r.get();
        if (e == null) {
            e = new TextElement(getBinding(element));
            synchronized (this) {
                swingElementMap.put(element, new WeakReference<javax.swing.text.Element>(e));
            }
        }
        return e;
    }
    
    private static final Element[] NO_CHILDREN = new Element[0];
    
    private class TextElement implements javax.swing.text.Element{
        private TextBinding    myBinding;
        private Element        myElement;
        
        public TextElement(TextBinding binding) {
            myBinding = binding;
            myElement = binding.getElement();
        }
        
        private PositionBounds getBounds() {
            return myBinding.getElementRange(false);
        }
        
        private Element[] getChildrenElements() {
            ElementOrder ck = myElement.getCookie(ElementOrder.class);
            if (ck == null)
                return NO_CHILDREN;
            return ck.getElements();
        }
        
        public int getElementIndex(int offset) {
            Element[] children = getChildrenElements();
            javax.swing.text.Element childElement;
            
            for (int i = 0; i < children.length; i++) {
                childElement = sourceToText(children[i]);
                if (childElement.getStartOffset() <= offset &&
                    //allow selection of elements at end of line.
                    childElement.getEndOffset() >= (offset-1) ) {
                    return i;
                }
            }
            return -1;
        }
        
        public javax.swing.text.AttributeSet getAttributes() {
            return null;
        }
        
        public Element getSourceElement() {
            return myElement;
        }
        
        public javax.swing.text.Document getDocument() {
            return ParserEnv.this.getDocument();
        }
        
        public javax.swing.text.Element getElement(int index) {
            Element[] els =  getChildrenElements();
            if (index > els.length)
                throw new IllegalArgumentException();
            return sourceToText(els[index]);
        }
        
        public int getElementCount() {
            return getChildrenElements().length;
        }
        
        public int getEndOffset() {
            return getBounds().getEnd().getOffset() - 1;
        }
        
        public String getName() {
            return myElement.getClass().getName();
        }
        
        public javax.swing.text.Element getParentElement() {
            Element parent = myElement.getParent();
            if (parent==null)
                return null;
            return sourceToText(parent);
        }
        
        public int getStartOffset() {
            return getBounds().getBegin().getOffset();
        }
        
        public boolean isLeaf() {
            return getChildrenElements().length == 0;
        }
    }


    //======================= SeqParser.Env ==================================//
    public org.openide.filesystems.FileObject getSourceFile() {
        return sdo.getPrimaryFile();
    }
    
    public String getSourceName() {
        return sdo.getPrimaryFile().getNameExt();
    }
    

    /** 
     * The implementation first tries to get a document - if it is already opened.
     * If it is, then it returns a CharArrayReader constructed on the document's
     * contents. Document is read using {@link StyledDocument#render} to avoid
     * data corruption.<P>
     * If the document is not (yet) opened, a {@link FilterReader} is
     * constructed so the output will not contain any '\r' characters.
     * <P>
     * The implementation does not handle IOException from lower layers in any
     * way and passes them to the caller.
     */
    public java.io.Reader getSourceText() throws java.io.IOException {
        CloneableEditorSupport ces = findEditorSupport();
        final StyledDocument doc = ces.getDocument();
        
        if (doc != null) {
            // can read from the document!
            final Segment s = new javax.swing.text.Segment();
            doc.render(new Runnable() {
                public void run() {
                    try {
                        doc.getText(0, doc.getLength(), s);
                    } catch (BadLocationException ex) {
                        // should never happen
                    }
                }
            });
            return new java.io.CharArrayReader(s.array, s.offset, s.count);
        } else {
            MultiDataObject.Entry me = sdo.getPrimaryEntry();
            String encoding = Util.getFileEncoding(me.getFile());
            return new FilterReader(me.getFile().getInputStream(),encoding);
        }
    }

    public void annotateThrowable(Throwable wrapper, Throwable nested) {
        Util.annotateThrowable(wrapper, nested);
    }    
    
    protected TextBinding getBinding(Element el) {
        ElementImpl impl = el.getCookie(ElementImpl.class);
        if (impl == null) {
            throw new IllegalArgumentException(
                    "Incompatible implementation on element: " + el); // NOI18N     
        }
        return (TextBinding)impl.getBinding();
    }
    
    /**
     * Used by SeqParser to create local SrcElement in the environement.
     */
    public SrcElement getLocalSrcElement(SrcElement.Impl sei) {
        throw new InternalError("Attempt to create LocalSrcElement");
    }    
    
    //======================= DocBinding.Env ===============================//    
    public CloneableEditorSupport findEditorSupport() {
        if (this.cloneableEditSupp == null) {
            bindEditorSupport(getEditor());
        }
        return cloneableEditSupp;
    }    
    
    /** Default behavior: return the start position; overide for other
     * behaviors.
     */
    public PositionRef findFreePosition(PositionBounds bounds) {
        StyledDocument doc = getDoc();
        if(doc==null) 
            return null;
        return bounds.getBegin();
    }
    
    public void takeLock() throws IOException {
        sdo.getPrimaryEntry().takeLock();
    }
    
    protected synchronized void bindEditorSupport(CloneableEditorSupport supp) {
        if (!editorBound) {
            supp.addPropertyChangeListener(this);
            documentLoaded = supp.isDocumentLoaded();
            editorBound = true;
            cloneableEditSupp = supp;
            if (documentLoaded) {
                this.propertyChange(new PropertyChangeEvent(supp, EditorCookie.Observable.PROP_DOCUMENT, null, null));
            }
        }
    }
        
    public Binding.Source createSrcBinding(SrcElement impl, SrcText txt) {
        return createSrcBindingFormat(impl,txt);
    }
    
    abstract protected Binding.Source createSrcBindingFormat(SrcElement impl, SrcText txt);
    
    public Binding.Seq createSeqBinding(SeqElement impl, SrcText txt) {
        return createSeqBindingFormat(impl,txt);
    }
    
    abstract protected Binding.Seq createSeqBindingFormat(SeqElement impl, SrcText txt);
    
    // ========= Implementation of Env ================//
    /**
     * Returns the document binding as the binding factory.
     */
    public BindingFactory getBindingFactory() {
        return docBinding;
    }

    public WrapperFactory getWrapperFactory() {
        return wrapper;
    }
    
    public <T extends Node.Cookie> T findElementCookie(Element el, Class<T> requestedClass) {
        return findCookie(el,requestedClass);
    }

    public <T extends Node.Cookie> T findCookie(Element el, Class<T> requestedClass) {
        return sdo.getCookie(requestedClass);
    }
    
    /**
     * Backfires PROP_COOKIE property change event in the source hierarchy.
     */
    protected void refireCookieChange(PropertyChangeEvent evt) {
        // fire the event on SrcElement.
        seqparser.fireElementPropertyChange(getSource(), evt);
        SrcElement.Impl impl = seqparser.getSourceImpl();
        if (impl == null)
            return;
        Element[] allseqlements = impl.getAllSeqElements();
        Collection<Element> allElements = new ArrayList<Element>(40);
        allElements.addAll(Arrays.asList(allseqlements));
        // FIRE!
	try {
	    disableCookieEvents = true;
    	    for (Iterator<Element> it = allElements.iterator(); it.hasNext(); ) {
    		seqparser.fireElementPropertyChange(it.next(), evt);
    	    }
	} finally {
	    disableCookieEvents = false;
	}
    }        
    
    public void suspendDocumentChanges() {
        suspended++;
    }
    
    public void resumeDocumentChanges() {
        --suspended;
    }
    
    protected boolean isDocumentSuspended() {
        return suspended > 0;
    }
    
    private void dissolve() {
        suspendDocumentChanges();
        sdo.removePropertyChangeListener(this);
        synchronized (this) {
            if (cloneableEditSupp != null) {
                cloneableEditSupp.removePropertyChangeListener(this);
                editorBound = false;
                StyledDocument d = cloneableEditSupp.getDocument();
                if (d != null)
                    d.removeDocumentListener(this);
            }
        }
        seqparser.invalidate();
    }
    
    /** Query whether the source needs to be parsed when closing the document 
     * in editor. Possibly important when closing without saving 
     * current modifications.
     */
    protected boolean parseOnClose() {
        return sdo.isValid() && sdo.isModified();
    }

    public void propertyChange(PropertyChangeEvent pce) {
        String evName = pce.getPropertyName();
        Object source = pce.getSource();
        
        if (source == sdo) {
            dataObjectPropertyChange(evName, pce);
        } else if (source instanceof SeqElement) {
            seqPropertyChange(evName, (SeqElement)source, pce);
        } else if (source instanceof SrcElement) {
            srcPropertyChange(evName, pce);
        } else if (source instanceof EditorCookie.Observable) {
            String prop = pce.getPropertyName();
            if( prop.equals(EditorCookie.Observable.PROP_DOCUMENT) ) {
                Runnable r = new Runnable() {
                    public void run() {
                        final StyledDocument doc = getDoc();
                        if (doc == null) {
                            removeDocListener();
                            if(parseOnClose())
                                seqparser.parse(SeqParser.PRIORITY_NORMAL, true, true);
                        }
                    }
                };
                RequestProcessor.getDefault().post(r); // posted to prevent deadlocking in multiview editor
            } else if( prop.equals(EditorCookie.Observable.PROP_MODIFIED) ) {
                documentchanged();
            } else if( prop.equals(EditorCookie.Observable.PROP_OPENED_PANES) ) {
                /* Do not invalidate */
            } 
        }
    }

    private synchronized void removeDocListener() {
        StyledDocument lastDoc = null;
        if (DocRef != null) {
            lastDoc = DocRef.get();
            lastDoc.removeDocumentListener(this);
            DocRef = null;
            documentLoaded = false;
        }
    }

    /** Return the document
     * and add a listener to it. Return null if no doc available.
     */
    private StyledDocument getDoc() {
        final StyledDocument doc = cloneableEditSupp.getDocument();
        StyledDocument lastDoc = null;
        synchronized (this) {
            if (DocRef != null) {
                lastDoc = DocRef.get();
            }
            if (lastDoc != doc) {
                if (lastDoc != null) {
                    lastDoc.removeDocumentListener(this);
                }
                if (doc != null ) {
                    doc.addDocumentListener(this);
                    DocRef = new WeakReference<StyledDocument>(doc);
                    documentLoaded = true;
                    // The goal of this forced parsing is to have annotations drawn
                    // Of coruse, this is unnecessary if the status is OK
                    if(seqparser.getStatus()!=SrcElement.STATUS_OK)
                        seqparser.parse(SeqParser.PRIORITY_NORMAL, true, true);
                } else {
                    DocRef = null;
                }
            }
        }
        return doc;
    }
        
    protected void dataObjectPropertyChange(String evName,PropertyChangeEvent pce) {
             if (DataObject.PROP_COOKIE.equals(evName)) {
		if (disableCookieEvents)
		    return;
                refireCookieChange(pce);
            } else if (DataObject.PROP_MODIFIED.equals(evName) && !sdo.isModified()) {
                // Unnecessary parsing after save or reload.
                // see above PropertyChange with EditorCookie.Observable.PROP_DOCUMENT
                //if (((DataObject)sdo).isValid())
                //    seqparser.parse(SeqParser.PRIORITY_NORMAL, false, true);
            } else if (DataObject.PROP_VALID.equals(evName) 
                && ((Boolean)pce.getNewValue()).booleanValue() == false) {
                dissolve();
            } 
    }
    
    protected void seqPropertyChange(String name, SeqElement seq, PropertyChangeEvent pce) {
        if (ElementProperties.PROP_VALID.equals(name)) {
            seq.removePropertyChangeListener(this);
        }

    }
    
    protected void srcPropertyChange(String name, PropertyChangeEvent pce) {
        if (ElementProperties.PROP_SEQELEMENT.equals(name)) {
            rescanSource((SrcElement)pce.getSource());
        }
    }

    /** Keep Listeners to SeqELements updated */
    protected void rescanSource(SrcElement el) {
        SeqElement[] seqs = (SeqElement[]) el.getAllElements();
        // sweep through all sequences.
        for (int i = 0; i < seqs.length; i++) {
            SeqElement seq = seqs[i];
            if (knownSeqs.add(seq)) {
                seq.addPropertyChangeListener(this);
            }
        }
    }
    
    //================= DocumentListener ============================
    public void changedUpdate(javax.swing.event.DocumentEvent e) {
        // text is not modified, so we can ignore this event
    }
    
    public void insertUpdate(javax.swing.event.DocumentEvent e) {
        documentchanged();
    }
    
    public void removeUpdate(javax.swing.event.DocumentEvent e) {
        documentchanged();
    }
    
    /** Called by Sibling Listener when documenet is modified;
     * simply reset parsing task and fire a change in parsing status.
     **/
    protected synchronized void documentchanged() {       
        if (!isDocumentSuspended()) {   //if suspended, block parsing
            seqparser.sourceChanged();
            restartTimer(); //This triggers a parsing on Document saving
        }
    }

    abstract protected void restartTimer(); 
    
}

