/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


/** Implement communication with DataObject.
 *
 * @author George Waldon
 */
package org.starbeans.modules.seqformat;

import java.beans.PropertyChangeListener;
import java.util.Collection;
import org.openide.filesystems.FileObject;
import org.openide.loaders.MultiDataObject.Entry;
import org.openide.nodes.Node;
import org.openide.text.CloneableEditorSupport;
import org.starbeans.modules.seqformat.parser.ParserMessage;
import org.starbeans.modules.seqformat.parser.Parsing;
import org.starbeans.modules.seqformat.source.SeqElement;
import org.starbeans.modules.seqformat.source.SrcElement;

public interface SeqDataObject {
    
    /** Get the primary entry.
    * @return the entry
    */
    public Entry getPrimaryEntry ();
    
    /** Get the primary file for this data object.
     */
    public FileObject getPrimaryFile ();
    
    /** Look for a cookie matching the requested class.
    *
    * @param type the class to look for
    * @return an instance of that class, or <code>null</code>.
    */
    public <T extends Node.Cookie> T getCookie (Class<T> type);
    
    /** Test whether the object is modified.
    * @return <code>true</code> if it is modified
    */
    public boolean isModified();
    
    /** Test whether the data object is still valid and usable.
     */
    public boolean isValid ();
    
    /** Get the parsed representation of this source file.
    * May not be fully parsed yet; the source element itself indicates its status.
    * @return the source element for this Java source file
    */
    public SrcElement getSource();

    /** Get the current editor support.
    */    
    public CloneableEditorSupport getEditor();
    
    /** Add a property change listener.
     * @param l the listener to add
    */
    public void addPropertyChangeListener (PropertyChangeListener l);
    
    /** Remove a property change listener.
     * @param l the listener to remove
    */
    public void removePropertyChangeListener (PropertyChangeListener l);
    
    /** Add the specific listener to the list of parsing listeners.
    * @param l listener to add
    */
    public void addParsingListener(Parsing.Listener l);
    
    /** Remove the specific listener from the list of parsing listeners.
    * @param l listener to remove
    */
    public void removeParsingListener(Parsing.Listener l);
    
    /** Fire the event for specified DataObject, normally this.
    */
    public void fireEvent(SeqDataObject sdo, Object hook, Collection<ParserMessage> messages);

    /** Communicate that a particular element has been selected.
     */
    public void fireElementSelectionChange(SeqElement el);
    
}
