/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.util.NbBundle;
import org.openide.filesystems.FileObject;

/** Miscellaneous utilities.
*
* @author Petr Hamernik, Ales Novak, George Waldon
*/
public final class Util extends Object {
    
    private static final Logger LOGGER = Logger.getLogger(Util.class.getName());

    public static boolean isLoggable(Class clazz) {
        return Logger.getLogger(clazz.getName()).isLoggable(Level.FINER);
    }

    public static void log(Class clazz, String message) {
        Logger.getLogger(clazz.getName()).finer(message);
    }

    static public void log(String msg) {
        LOGGER.fine(msg);
    }
    
    /** Computes the localized string for the key.
    * @param key The key of the string.
    * @return the localized string.
    */
    static String getString(String key) {
        return NbBundle.getMessage(Util.class, key);
    }
    
    static void annotateThrowable(Throwable t, Throwable nested) {
        t.initCause(nested);
    }
    
    static final String ATTR_FILE_ENCODING = "Content-Encoding"; // NOI18N

    /**
     * Retrieves file's encoding. If the file does not specify any encoding,
     * null is returned. Note: an empty string is also a legal value, which also
     * means platform-default encoding.
     * @param someFile file object to retrieve encoding for
     * @return encoding string, or null if no explicit encoding is given.
     */
    static String getFileEncoding0(FileObject someFile) {
        return (String)someFile.getAttribute(ATTR_FILE_ENCODING);
    }
    
    /**
     * Retrieves file's encoding.
     * @param someFile file object to retrieve encoding for
     * @return encoding string, or null if no explicit encoding is given in file's
     * or IDE's settings.
     */
    static String getFileEncoding(FileObject someFile) {
        String enc = getFileEncoding0(someFile);
        if ("".equals(enc))
            return null;
        else
            return enc;
    }
}
