/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.starbeans.modules.seqformat.model;

import java.beans.PropertyChangeEvent;
import java.util.Collection;
import java.util.EventListener;
import java.util.Set;
import java.util.Map;

import org.starbeans.modules.seqformat.source.SrcException;

/**
 * The interface defines a listener that listens for outcome of atomic operations
 * on the model. CommitListeners are contacted before the atomic operation completes
 * and are given a chance to abort the whole operation. The commit is simple two-phase
 * one; in the first phase, all listeners are contacted and queried to confirm 
 * the change integrity. In the second phase, if no listener aborts the operation,
 * the listeners are notified that the operation completed successfully. There is
 * no notification that the operation was aborted, currently.
 *
 * @author  Svatopluk Dedic, George Waldon
 */
public interface CommitListener extends EventListener {

    /**
     * Notifies the listener about changes that are made to the model. The method
     * is only a notification message and can not abort the operation, for that
     * purposes, use {@link #queryCommitChanges} instead.<P>
     * The method is called just before the write lock on the model is released.
     * Avoid lengthy operations from within this method, use RequestProcessor, if necessary.
     * Recipients are not allowed to alter the source model.
     * @param created set of elements that were created within the atomic operation.
     * @param removed set of elements that were removed during the operation
     * @param changed map indexed by elements that were changed during the operation. 
     * values of the map are elements of the same type containing saved state of
     * the changed element.
     */
    public void changesCommited(Set created, Set removed, Map changed, Map<ElementImpl, Collection<PropertyChangeEvent>> sourcemap);

    public interface Veto extends CommitListener {

        /**
         * Asks the listener whether it is satisfied with the changes made so far.
         * Recipients are not allowed to alter the source model.
         * @param created set of elements that were created within the atomic operation.
         * @param removed set of elements that were removed during the operation
         * @param changed map indexed by elements that were changed during the operation. 
         * values of the map are elements of the same type containing saved state of
         * the changed element.
         * @param zero, if the operation is going out of the write lock; if zero, this is
         * the last chance to prevent the modification from being done.
         * @throws SourceException if the listener is not satisfied with impact of the
         * changes on the model.
         */
        public void queryCommitChanges(Set created, Set removed, Map changed,
                int lockLevel)
                throws SrcException;
    }
}
