/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.model;

import java.beans.PropertyChangeEvent;
import org.starbeans.modules.seqformat.source.SrcException;

/**
 *
 * @author  sdedic, George Waldon
 */
public interface ElementEvents {
    /** Adds a PropertyChangeEvent to be fired. The event is not necessarily fired
     * immediately, but may be queued and fired after a lock on the element/model is
     * released. The method may throw IllegalArgumentException, if the event source
     * reported in the event does not match the return value of {@link #getSource}
     * @param evt the event to be fired
     */
    public void addPropertyChange(PropertyChangeEvent evt);
    
    /** Fires a property change event to Vetoable listeners. If a listener cancels
     * the ongoing change, its response is wrapped into a SourceException object which
     * is thrown from the method. The method may throw IllegalArgumentException, if the event source
     * reported in the event does not match the return value of {@link #getSource}
     * @param evt the event to be fired
     */
    public void fireVetoableChange(PropertyChangeEvent evt) throws SrcException;
    
    /** Adds another UndoableEdit. Generally there should be an edit for each partial
     * operation on any data structure. Those edits are grouped such that edits generated
     * during one particular change operation are always undone as a group.
     * @param edit the edit that should be queued in the undo queue
    public void addUndoableEdit(UndoableEdit edit);
     */
    
    /** Returns the event source for events that should be fired on this interface.
     * @return object representing the event source.
     */
    public Object getEventSource();
    
    /** Returns the implementation of the element.
     */
    public ElementImpl getElementImpl();
}
