/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.starbeans.modules.seqformat.model;

import java.beans.PropertyChangeEvent;
import org.starbeans.modules.seqformat.source.Element;
import org.starbeans.modules.seqformat.source.Identifier;
import org.starbeans.modules.seqformat.source.SeqElement;
import org.starbeans.modules.seqformat.source.SrcException;

/**
 *
 * @author  George Waldon
 */
public abstract class SeqElementImpl extends ElementImpl implements SeqElement.Impl {
    
    static final long serialVersionUID = 200401311223000L;
    
    /** Name of this element. 
     *  
     */
    protected Identifier  name;
    
    /** Link to the source element.
     */
    private SrcElementImpl       srcImpl;  
        
    public SeqElementImpl(DefaultSeqModel model) {
        super(model);
    }
    
    // ->DefaultSeqModel->SrcTxt
    protected final Binding createBinding(Element el) {
        return (Binding) getModelImpl().getBindingFactory().bindSeq((SeqElement)el);
    }

    /** Retrieves the binding object.
     */
    protected final Binding.Seq getSeqBinding() { 
        return (Binding.Seq)getBinding();
    }
   
    protected void createFromModel(Element model) throws SrcException {
        Identifier name = ((SeqElement) model).getName();
        setName((Identifier) name.clone());
    }   
    
    protected SrcElementImpl findSource() {
        return this.srcImpl;
    }
     
    protected boolean parentValid() {
        return srcImpl.isValid();
    }
    
    public Object readResolve() {
        return new SeqElement(this,null);
    }

    protected void setParent(ElementImpl parent) {
        if (parent instanceof SrcElementImpl) {
            srcImpl = (SrcElementImpl)parent;
        }
    }      
    
    /**
     * Implementation of name change for an Element. The implementation will:<UL>
     * <LI>check vetoable listener acknowledges of the change
     * <LI>ask the Binding for performing the change
     * <LI>fire the PropertyChangedEvent.
     * @param id new name to use with the element.
     */
    public void setName(Identifier id) throws SrcException {
        Object token = takeLock();
        try {
            CheckName(id);
            commit();
        } finally {
            releaseLock(token);
        }
    }
    
    protected void CheckName(Identifier id) throws SrcException {
        PropertyChangeEvent evt;
        if (!isCreated()) {
            Identifier ex = this.name;
            if (ex != null && ex.compareTo(id)==0)
                return;
            evt = new PropertyChangeEvent(getElement(),PROP_NAME,ex,id);
            checkVetoablePropertyChange(evt);
            changeName(id);
            fireOwnPropertyChange(evt);
        }
        this.name = id;
    }
    
    /**
     * Handles out the name change. 
     */
    protected void changeName(Identifier id) throws SrcException {
        Identifier old = getName();        
        try {
            getSeqBinding().changeName(id); 
        } catch (SrcException sex) {
            getSeqBinding().changeName(old);
            throw sex;
        }        
    }
    
    /** Getter for name.
    * @return the name
    */
    public Identifier getName() {
        return name;
    }
}
