/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.starbeans.modules.seqformat.model;

import java.lang.ref.*;
import java.util.*;

import org.starbeans.modules.seqformat.source.*;

/**
 * Represents a source file.  <B>This element has not any
 * Binding</B> since it has no (direct) properties.<P>
 * Although this implementation *does* implement entire SrcElement.Impl interface,
 * methods that have not anything to do with the model itself are <B>dummy</B> and
 * do nothing (parse, getStatus). 
 * @author  sdedic, George Waldon
 */
public abstract class SrcElementImpl extends ElementImpl implements SrcElement.Impl,
    ElementOrder {
    
    private static final long serialVersionUID = 200311210543000L;
        
    public SrcElementImpl(DefaultSeqModel model) {
        super(model);
    }
    
    protected void notifyElementCreated() {
        // do nothing -- override ElementImpl's behaviour.
    }
    
    protected void createFromModel(Element model) throws SrcException {
        // no-impl
    }
    
    protected Binding createBinding(Element el) {
        return null;
    }
 
    //================ Implementation of SrcElement.Impl ====================//
    abstract public boolean acceptSeqElement(SeqElement sel);
    
    abstract public SeqElement[] getAllSeqElements();
    
    // Element change is implemented but subclasses.
    abstract public void changeSeqElements(SeqElement[] els, int mode) throws SrcException;

    //=================== Implementation of ElementOrder =====================//
    /** Implementation in subclass */
    public Element[] getElements() {    
        throw new InternalError("Attempt to call getElements"); // NOI18N
    }
    
    public void notifyCreate() {
        super.notifyCreate();
    }
    
    public void notifyRemove() {
        super.notifyRemove();
    }
    
    private Binding.Source getSourceBinding() {
        return (Binding.Source)getBinding();
    }

    /** Dummy method to satisfy typing requirements. This method does not work with the
     * model, but rather serves as interface to the parser. The method may be implemented
     * in subclasses or delegating implementations.
     */
    public int getStatus() {
        return SrcElement.STATUS_NOT;
    }
    
    public org.openide.util.Task prepare() {
        return org.openide.util.Task.EMPTY;
    }
    
    public void runAtomic(Runnable run) {
        // no-op
    }
    
    public void runAtomicAsUser(Runnable run) throws SrcException {
        // no-op, see proxy implementation.
    }
 
    public abstract void updateMembers(String name, Element[] elements, 
    int[] orderIndices, int[] optMap);
    
    // Implementation details
    ///////////////////////////////////////////////////////////////////////////////////
    public void setParent(ElementImpl impl) {
        // no-op, sources cannot have parents.
    }
    
    protected SrcElementImpl findSource() {
        return this;
    }

    public Object readResolve() {
        return null;
    }
    
    protected Element cloneSelf() {
        return null;
    }
    
    protected boolean parentValid() {
        return isValid();
    }

    
}
