/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.beans.PropertyChangeEvent;
import java.util.*;
import javax.swing.text.StyledDocument;

import org.openide.text.*;
import org.openide.cookies.LineCookie;

/**
 *
 * @author Petr Hamernik, George Waldon
 */
public class AnnotationSupport {
     
    static final long serialVersionUID = 200312221132000L; 
    
    /* List of annotations attached to this support */
    private ArrayList<ParserAnnotation> annotations=new ArrayList<ParserAnnotation>();
    
    private int maxErrors;
    
    private DataEditorSupport ed;
    
    public AnnotationSupport(DataEditorSupport des, int maxerrors) {
        ed = des;
        maxErrors = maxerrors;
    }
    
    public boolean processAnnotations(ParserMessage[] errors) {
        ArrayList<ParserAnnotation> added,removed,unchanged;
        Iterator i;
        Collection<ParserAnnotation> newAnnotations;
        if (errors==null) {   
            // request processed by someone else
            return false;
        }
        newAnnotations = getAnnotations(errors);
        added=new ArrayList<ParserAnnotation>(newAnnotations);
        added.removeAll(annotations);
        unchanged=new ArrayList<ParserAnnotation>(annotations);
        unchanged.retainAll(newAnnotations);
        removed=annotations;
        removed.removeAll(newAnnotations);
        detachAnnotations(removed);
        if (!added.isEmpty() && ed.isDocumentLoaded()) {

            // Partial fix of #33165 - document read-locking
            final ArrayList finalAdded = added;
            StyledDocument doc = ed.getDocument();
            Runnable docRenderer = new Runnable() {
                public void run() {
                    LineCookie cookie = ed.getDataObject().getCookie(LineCookie.class);
                    Line.Set lines = cookie.getLineSet();

                    for (Iterator i=finalAdded.iterator();i.hasNext();) {
                        ParserAnnotation ann=(ParserAnnotation)i.next();

                        ann.attachToLineSet(lines);
                    }
                }
            };

            if (doc != null) {
                doc.render(docRenderer);
            } else {
                docRenderer.run();
            }
        }        
 
        annotations=unchanged;
        annotations.addAll(added);
        return true;
    }

    /** @return annotations for the given list of errors */
    private Collection<ParserAnnotation> getAnnotations(ParserMessage[] errors) {
        HashMap<Integer,ParserAnnotation> map = new HashMap<Integer,ParserAnnotation>(2*errors.length);
        for (int i = 0; i < errors.length; i++) {
            ParserMessage err=errors[i];
            int line = err.getLineNumber();
            
            // annotate only errors with positive line number
            if (line>0) {
                int column = err.getColumn();
                int length = err.getLength();
                String message = err.getDescription();
                ParserAnnotation anno = 
                    new ParserAnnotation(err.getSeverity(),line, column, length, message);

                // This is trying to ensure that annotations on the same
                // line are "chained" (so we get a single annotation for
                // multiple errors on a line).
                // If we knew the errors were sorted by file & line number, 
                // this would be easy (and we wouldn't need to do the hashmap
                // "sort"
                Integer lineInt = new Integer(line);
                ParserAnnotation prev = map.get(lineInt);
                if (prev != null) {
                    prev.chain(anno);
                } else if (map.size() < maxErrors) {
                    map.put(lineInt, anno);
                }
            }
        }
        return map.values();
    }
    
    public int getSize() {
        return annotations.size();
    }
    
    public void setMaxErrors(int newsize) {
        maxErrors = newsize;
    }
    
    public void clearSupport() {
        detachAnnotations(annotations);
        annotations.clear();
    }
    
    
    private static void detachAnnotations(Collection anns) {
        Iterator i;

        for (i=anns.iterator();i.hasNext();) {
            Annotation ann=(Annotation)i.next();
            if (ann.getAttachedAnnotatable() != null) {
                ann.detach();
            }
        }
    }    
}
