/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.util.*;
import org.openide.nodes.Node;
import org.openide.text.PositionBounds;
import org.starbeans.modules.seqformat.source.*;
import org.starbeans.modules.seqformat.model.SeqModel;
import org.starbeans.modules.seqformat.srcgen.DocBinding;
import org.starbeans.modules.seqformat.srcgen.TextBinding;

/**
 *
 * @author  svata, George Waldon
 * @version 
 */
public abstract class BaseElementInfo implements ElementFactory.Item {
    ElementFactory.Item parent;
    PositionBounds      wholeBounds;
    PositionBounds      bodyBounds;
    PositionBounds      headerBounds;
    
    Identifier          name;
    Element             el;
    TextBinding         binding;
    
    BaseElementInfo(Identifier name) {
        this.name = name;
    }
    
    public Identifier getName() {
        return name;
    }
    
    public void setParent(ElementFactory.Item efi) {
        parent = efi;
    }
    
    public void setWholeBounds(PositionBounds pb) {
        wholeBounds = pb;
    }
    
    public void setHeaderBounds(PositionBounds pb) {
        headerBounds = pb;
    } 
    
    public void setBodyBounds(PositionBounds pb) {
        bodyBounds = pb;
    }
    
    // Called by ChildCollection->updateChildren()
    public Element createModelElement(Element parent, SeqModel.Updater model) 
    throws SrcException {
        Element el = createModelImpl(model, parent);    //see implemention in subclasses
        updateElement(model, el);
        return this.el;
    }
    
    protected abstract Element createModelImpl(SeqModel.Updater model, Element parent);
    
    public void updateElement(SeqModel.Updater model, Element el) throws SrcException {
        this.el = el;
        this.binding = (TextBinding)model.getElementBinding(el);
    }
    
    public int comparePositionTo(TextBinding b) {
        if (b == null)
            return -1;
        // If DefRange is true, comparison is done with bodybounds, if not with wholebounds
        PositionBounds bounds = b.getElementRange(true);
        
        int begin = wholeBounds.getBegin().getOffset();
        int end = wholeBounds.getEnd().getOffset();
        
        int otherBegin = bounds.getBegin().getOffset();
        int otherEnd = bounds.getEnd().getOffset();
        
        if (otherBegin == otherEnd)
            // the binding is invalid - extinct ??
            return -1;
        
        if (begin < otherEnd && end > otherBegin)
            // MATCH!!
            return 0; 
        
        if (end == otherBegin)
            return -1;
        
        return end - otherBegin;
    }
    
    public Node.Cookie getCookie(Class desiredClass) {
        return null;
    }

    public void updateBindings(DocBinding updater) {
        updateBinding(updater, binding);
    }
    
    public void updateBinding(DocBinding updater, TextBinding b) {
        b.updateBounds(TextBinding.BOUNDS_ALL, wholeBounds);
        b.updateBounds(TextBinding.BOUNDS_HEADER, headerBounds);
        b.updateBounds(TextBinding.BOUNDS_BODY, bodyBounds);
    }
    
    protected static void updateChildren(DocBinding updater,
        Collection infos, TextBinding.Container container) {
        List blist = new ArrayList(infos.size());
        for (Iterator it = infos.iterator(); it.hasNext(); ) {
            BaseElementInfo i = (BaseElementInfo)it.next();
            i.updateBindings(updater);
            blist.add(i.binding);
        }
        container.updateChildren(blist);
    }

    public static class NameFinder extends ElementMatch.AbstractFinder {
        protected boolean matches(BaseElementInfo info, Element old) {
            SeqElement oldMember = (SeqElement)old;
            return info.name.equals(oldMember.getName());
        }
    }
}
