/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.util.*;

import org.starbeans.modules.seqformat.source.*;
import org.starbeans.modules.seqformat.model.SeqModel;

/**
 *
 * @author  sdedic, George Waldon
 * @version 
 */
public class ChildCollection extends Object {
    Collection<BaseElementInfo> children;
    ElementMatch.Finder[]   finders; 
    int []                  ids;
    int []                  results;
    int                     idIndex;
    Class                   componentClass;
    
    private static final int MIN_IDS = 15;
    
    public ChildCollection(ElementMatch.Finder[] finders, Class componentClass) {
        this.finders = finders;
        this.componentClass = componentClass;
    }
    
    public int[] getIDs() {
        return ids;
    }
    
    public int[] getResultMap() {
        return results;
    }
    
    protected Element[] createArray(int size) {
        return (Element[])java.lang.reflect.Array.newInstance(componentClass, size);
    }
    
    public void addChild(BaseElementInfo el, int id) {  // id = index in the allmembers original collection
        if (children == null) {
            children = new LinkedList<BaseElementInfo>();
            if (id != -1)
                ids = new int[MIN_IDS];
        }
        children.add(el);
        if (ids != null) {
            if (ids.length == idIndex) {
                int[] newIDs = new int[ids.length * 2];
                System.arraycopy(ids, 0, newIDs, 0, ids.length);
                ids = newIDs;
            }
            ids[idIndex++] = id;
        }
    }
    
    private Element[] createAllChildren(Element parent, SeqModel.Updater updater) 
        throws SrcException {
        Element[] newElements = createArray(children.size());
        int i = 0;
        for (Iterator<BaseElementInfo> it = children.iterator(); it.hasNext(); i++) {
            BaseElementInfo info = it.next();
            newElements[i] = info.createModelElement(parent, updater);
        }
        return newElements;
    }

    public Element[] updateChildren(Element parent, SeqModel.Updater model, Element[] oldChildren)
    throws SrcException {
        if (oldChildren.length == 0) {
            return createAllChildren(parent, model);
        }
        ElementMatch match = new ElementMatch(oldChildren, this.children);
        for (int i = 0; i < finders.length; i++) {
            match.reset();
            finders[i].findMatches(match);
        }

        int[] mapping = match.getResultMap();
        Element[] oldElements = match.getOldElements();
        Element[] newElements = createArray(children.size());
        Collection current = match.getInfos();
        Iterator it = current.iterator();
        results = mapping;
        
        for (int i = 0; it.hasNext(); i++) {
            BaseElementInfo info = (BaseElementInfo)it.next();
            if (mapping == null || mapping[i] == -1) {
                // create the child element within its parent
                newElements[i] = info.createModelElement(parent, model);
            } else {
                newElements[i] = oldElements[mapping[i]];
                info.updateElement(model, newElements[i]);
            }
        }
        return newElements;
    }
    
    public void mapChildren(Element[] n, Element[] map) {
        for (int i = 0; i < n.length; i++) {
            map[ids[i]] = n[i];
        }
    }
    
    public Collection getChildren() {
        return this.children;
    }
}
