/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.util.BitSet;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import org.starbeans.modules.seqformat.source.Element;
import org.starbeans.modules.seqformat.model.SeqModel;

/**
 *
 * @author  svata, George Waldon
 * @version 
 */
public class ElementMatch {
    Element[]   oldElements;
    BitSet      usedElements;
    BitSet      matchedInfos;
    Collection  currentInfos;
    int         currentIndex;
    int         matchedCount;
    SeqModel.Updater model;

    Enumeration     enume;
    int[]           resultMap;
    BaseElementInfo nextElem;
    
    public ElementMatch(Element[] oldElements, Collection currentInfos) {
        this.currentInfos = currentInfos;
        this.oldElements = oldElements;
        this.matchedInfos = new BitSet(currentInfos.size());
        this.usedElements = new BitSet(oldElements.length);
        resultMap = new int[currentInfos.size()];

        java.util.Arrays.fill(resultMap, -1);
    }
    
    public int[] getResultMap() {
        return resultMap;
    }
    
    public void reset() {
        enume = Collections.enumeration(currentInfos);
        currentIndex = -1;
        nextElem = null;
    }
    
    public Element[] getOldElements() {
        return oldElements;
    }
    
    public boolean hasMoreElements() {
        if (matchedCount == currentInfos.size())
            return false;
        if (nextElem != null)
            return true;
        
        while (true) {
            if (!enume.hasMoreElements())
                return false;
            ++currentIndex;
            BaseElementInfo i = (BaseElementInfo)enume.nextElement();
            if (!matchedInfos.get(currentIndex)) {
                nextElem = i;
                break;
            }
        }
        return nextElem != null;
    }
    
    public BaseElementInfo nextElement() {
        BaseElementInfo ret = nextElem;
        nextElem = null;
        return ret;
    }
    
    public Collection getInfos() {
        return currentInfos;
    }
    
    public boolean isUsed(int elementIndex) {
        return usedElements.get(elementIndex);
    }
    
    public void matchElement(int elementIndex) {
        matchedInfos.set(currentIndex);
        usedElements.set(elementIndex);
        resultMap[currentIndex] = elementIndex;
        matchedCount++;
    }
    
    public interface Finder {
        public void findMatches(ElementMatch data);
    }
    
    public static abstract class AbstractFinder implements Finder {
        
        public void findMatches(ElementMatch data) {
            Element[] elems = data.getOldElements();
            
            while (data.hasMoreElements()) {
                BaseElementInfo info = data.nextElement();
                for (int i = 0; i < elems.length; i++) {
                    if (data.isUsed(i))
                        continue;
                    if (matches(info, elems[i])) {
                        data.matchElement(i);
                        break;
                    }
                }
            }
        }
        
        protected abstract boolean matches(BaseElementInfo info, Element old);
    }
}
