/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.io.IOException;
import java.util.Collection;

/** Request for parsing the source. This interface defines interactions
    between the parsing engine and the request object.
    Parser engine takes appropriate DataObject from the request and processes
    its file. The engine consults the request about its validity from time to 
    time so it can stop time-consuming processing as soon as possible.
    The main purpose is to inform the Request about states of processing so it
    can properly track its validity.
*
* @author Petr Hamernik
*/
public interface ParseObjectRequest {
    /** Sets the number of syntax errors that occured during parsing.
    */
    public void setSyntaxErrors(int errors);
    
    /** Returns the number of syntax errors.
    */
    public int  getSyntaxErrors();
    
    /** 
     * Sets number of semantic errors.
     */
    public void setSemanticErrors(int errors);
    
    /** Returns Element implementation creation factory. The Engine asks for the
        factory after it finishes general text scan. The factory will be responsible
        for creating the source text representation and to bind it to the request.
    */
    public ElementFactory getFactory();
    
    /** 
     * Called from the parser to obtain data to be parsed. 
     * @throws IOException if the data can't be returned.
    */
    public char[] getSource() throws IOException;

    /* Access from the parser to local environement*/
    public SeqParser.Env getEnvironment();
    
    /** Determines if the request is still valid.
        In general, request processing stops whenever this method returns false.
        It is <B>required</B> that implementation returns false, if the source contents
        changes during request processing.
        Implementation might return false whenever it decides to discard potential results
        from this request to provide hint to the Parser Engine.
    */
    public boolean isValid();

    public boolean needsProcessing();
    
    /** Notifies the request that the parser engine is about to process the request.
        Should any data passed to the parser engine change after this call, the
        parser request implementation should invalidate the request.
    */
    public void notifyStart();

    /** Notifies the request object that the engine has finished its processing.
        The request can then become completed (depending on its validity)
    */
    public void notifyComplete();
     
    /** @return the set of errors encountered while processing this
     * request */
    public Collection<ParserMessage> getMessages();
   
    public ErrConsumer getErrConsumer();
    
    public org.openide.filesystems.FileObject  getSourceFile();  
        
    public String getSourceName();

}
