/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.io.IOException;
import org.openide.nodes.Node;
import org.openide.cookies.*;
import org.openide.filesystems.FileObject;

/** Simple interface to a parser engine
 *  The engine should process passed source file(s) and give out enough information
 *  to produce a model for the source. 
 *
 * @author  sdedic, George Waldon
 * @version 
 */
public interface ParserEngine extends Node.Cookie {
    /** Registers a new request with the parser engine.
        When a request is registered, it can be processed at any time. The engine
        can carry this task during processing of other requests.
     * @param r Request to be registered
     * @param fo FileObject that the request should be bound to. The request will
     * be processed if the parser/compiler touches `fo' during other processing.
    */
    public void register(ParseObjectRequest r, FileObject fo);

    /**
     * Unregisters a request filed for the given FileObject. 
     */
    public void unregister(FileObject fo);

    /** Determines if a parse request is already registered with the engine.
    */
    public boolean isRegistered(ParseObjectRequest r);

    /** Clears all requests in the parser engine.
    */
    public void unregisterAll();

    /** Process the request immediately. The method returns after the request is
        completed (either successfully or with some errors).
        If request was already completed, it is processed again yielding fresh
        results.
    */
    public void process(ParseObjectRequest r) throws IOException;
}
