/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;


/*
 * This class represents a single error encountered when parsing
 * a source file. 
 * @author Tor Norbye, George Waldon
 */

class ParserMessageImpl extends ParserMessage {
    private final int line;
    private final int column;
    private final int length;
    private final Object severity;
    private final String message;

    /** Construct new ParserMessage.
     * @param file The file where the error occurred.
     * @param line The line number where the error occurred
     * @param column The column within the line where the error
     *    occurred.
     * @param length The length of the error
     * @param message Messages from the compiler describing
     *    the error
     */
     ParserMessageImpl(Object sev, int l, int col, int len, String msg) {
        severity = sev;
        line = l;
        column = col;
        length = len;
        message = msg;
    }

    /** Return the line number on which the error occurred.
     *  @return the line number on which the error occurred.
     */
    public int getLineNumber() {
        return line;
    }

    /** Return the severity on which the error occurred.
     *  @return the severity on which the error occurred.
     */
    public Object getSeverity() {
        return severity;
    }
    
    /** Return the column number within the line on which the error occurred.
     *  @return the column number within the line on which the error occurred.
     */
    public int getColumn() {
        return column;
    }
    
    /** Return the length of the error.
     *  @return the number of character encmpassed by the error.
     */
    public int getLength() {
        return length;
    }
    
    /** Return the text description of the error
     * @return the text description of the error
     */
    public String getDescription() {
        return message;
    }

    /** For debugging only */
    public String toString() {
        return line + ":" + column + ":" + message; // NOI18N
    }
}

