/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.util.Collection;
import java.util.EventListener;
import java.util.EventObject;
import org.starbeans.modules.seqformat.SeqDataObject;

/** The public interface for parsing of sources. This class contains listener
* interface and the event class, also the methods for registration and unregistration
* of the listener.
*
* @author Petr Hamernik, George Waldon
*/
public class Parsing extends Object {


    /** Fire the event for the specified SeqDataObject.
    */
    public static void fireEvent(SeqDataObject sdo, Object hook, 
                          Collection<ParserMessage> messages) {
        sdo.fireEvent(sdo,hook, messages);
    }

    /** The listener interface for everybody who want to control all
    * parsed DataObjects.
    */
    public static interface Listener extends EventListener {
        /** Method which is called everytime when some object is parsed.
        * @param evt The event with the details.
        */
        public void objectParsed(Event evt);
    }
    
    /** Parsable interface for documents which can be parsed */
    public static interface Parsable {
        public void parseDocument(int priority,boolean ignoreClean);
    }

    /** The event class used in Listener.
    */
    public static class Event extends EventObject {
        static final long serialVersionUID =200311110914000L;
        private final Collection<ParserMessage> messages;
        private final Object hook;
        
        /** Construct the new Event.
        * @param do DataObject which is the source of the event.
        * @param hook the object which prevents the garbage collector remove the parsing
        * @param messages the error messages from the parser
        *   information till this event lives.
        */
        public Event(SeqDataObject sdo, Object h, Collection<ParserMessage> msg) {
            super(sdo);
            messages = msg;
            hook = h;
        }

        /** @return the errors encountered for this source. May be
            null if there are no associated errors. */
        public ParserMessage[] getMessages() {
            if (messages == null) {
                return null;
            }
            ParserMessage[] copy = new ParserMessage[messages.size()];
            messages.toArray(copy);
            return copy;
        }

    }
}
