/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.parser;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import java.util.List;
import java.util.Iterator;
import java.util.LinkedList;
import org.openide.cookies.SaveCookie;
import org.openide.text.CloneableEditorSupport;
import org.starbeans.modules.seqformat.SeqDataObject;

/** Miscellaneous utilities.
*
* @author Petr Hamernik, Ales Novak
*/
public final class Util extends Object {
    
    public static char[] readContents(Reader r) throws IOException {
        int read = 0;
        int total = 0;
        int offset;
        char[] buffer;
        List buflist = new LinkedList();

        do {
            buffer = new char[2048];
            offset = 0;
            while (offset < buffer.length) {
                read = r.read(buffer, offset, buffer.length - offset);
                if (read == -1) break;
                offset += read;
            }
            if (offset > 0) buflist.add(buffer);
            total += offset;
        } while (read >= 0);
        r.close();

        buffer = new char[total];
        Iterator it = buflist.iterator();
        int offset2 = 0;
        while (it.hasNext()) {
            char[] buf = (char[])it.next();
            int size = (it.hasNext()) ? buf.length : total - offset2;
            System.arraycopy(buf, 0, buffer, offset2, size);
            offset2 += size;
        }
        return buffer;
    }
    
    /** Returns contents of fileobject fo. If the object is opened in the editor,
      the function returns current contents of the edited document. In that case,
      if <b>save</b> is true, the editor content is saved.
      If the file is not opened in the Editor, it is read from the disk.
      @return contents of the file/editor document;.
    */
    public static char[] getContent(FileObject fo, boolean save, boolean filter, String encoding) throws IOException {
        DataObject obj = DataObject.find(fo);
        CloneableEditorSupport editor = null;

        if (obj instanceof SeqDataObject)
            editor = ((SeqDataObject) obj).getEditor();

	final Document doc;
        if ((editor != null) && (doc = editor.getDocument()) != null) {
            // loading from the memory (Document)
            final String[] str = new String[1];
            // safely take the text from the document
            Runnable run = new Runnable() {
                               public void run() {
                                   try {
                                       str[0] = doc.getText(0, doc.getLength());
                                   }
                                   catch (BadLocationException e) {
                                       // impossible
                                   }
                               }
                           };
            if (save) {
                SaveCookie cookie = obj.getCookie(SaveCookie.class);
                if (cookie != null) {
                    cookie.save();
                }
            }
            doc.render(run);
            return str[0].toCharArray();
        } else {
            // loading from the file
            InputStream is = new BufferedInputStream(fo.getInputStream());
            Reader reader;
            if (encoding == null) {
                reader = new InputStreamReader(is);
            } else {
                reader = new InputStreamReader(is, encoding);
            }
            
            return readContents(reader);
        }
    }
    
}
