/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.source;

/** Prints elements in a textual form.
*
*   Constants to indicate events occurring while an element is printed,
*   e.g. {@link #ELEMENT_BEGIN} and {@link #ELEMENT_END}.
*
*   A Mark method which notify about notable points inside the elements.
*   For example, if a printer implementation wants to print only
*   the header, it may implement {@link #Mark} to throw
*   {@link ElementPrinterInterruptException} to stop the printing there
*   when it encounters {@link #HEADER_END}.
*
* @author Petr Hamernik, George Waldon
*/
public interface ElementPrinter {

    /* Bound types - Simplest representation, each element contains a header 
     * and a body. The header and the body are separated by a new line.
     * More complex representation should use subclasses */
    
    /** Beginning of whole element. */
    public static final int ELEMENT_BEGIN = 0;
    /** End of whole element. */
    public static final int ELEMENT_END = 1;

    /** Beginning of name. */
    public static final int HEADER_BEGIN = 2;
    /** End of name. */
    public static final int HEADER_END = 3;

    /** Beginning of body. */
    public static final int BODY_BEGIN = 4;

    /** End of body. */
    public static final int BODY_END = 5;
    
    
    /** Print some text.
    * @param text the text
    * @exception ElementPrinterInterruptException
    */
    public void print(String text) throws ElementPrinterInterruptException;

    /** Print a line of text with a newline.
    * @param text the text
    * @exception ElementPrinterInterruptException
    */
    public void println(String text) throws ElementPrinterInterruptException;
    
    /** Mark a notable point in an element.
    * @param element the element
    * @param what which point (a bound type!)
    * @exception ElementPrinterInterruptException
    */
    public void mark(Element element, int what) throws ElementPrinterInterruptException;
}
