/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.source;


/** Type element to represent a sequence. Local subclasses bear the actual sequence.
 *
 * @author  George Waldon
 */
public class SeqElement extends Element implements Cloneable  {

    static final long serialVersionUID = 200401311143000L;   

    /** SrcElement containig this element. A source
     * contains one or more sequences.
     */
    private SrcElement source;        


    /** Create a new SeqElement in memory.
    */
    public SeqElement() {
        this(new Memory (), null);
    }
    
    public SeqElement(Element.Impl impl, SrcElement source) {
        super(impl);
        this.source = source;
    }

    /** Return the parent Element to which this Element belongs 
     */
    public Element getParent() {
        return source;
    }

    /** Get the source element
    */
    public SrcElement getSource () {
        return source;
    }
    
    /** @return implemetation
    */
    protected Impl getSeqImpl () {
        return (Impl)impl;
    }

    /** Get the name of this member.
    * @return the name
    */
    public final Identifier getName() {
        return getSeqImpl().getName();
    }

    /** Set the name of this member.
    * @param name the name
    * @throws SrcException if impossible
    */
    public void setName(Identifier name) throws SrcException {
        getSeqImpl().setName(name);
    } 

    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }
    
    /* Print this element to an element printer.
    * @param printer the printer to print to
    * @exception ElementPrinterInterruptException if the printer cancels the printing
    */    
    public void print(ElementPrinter printer) throws ElementPrinterInterruptException {
        printer.mark(this, ElementPrinter.ELEMENT_BEGIN);
        printer.mark(this, ElementPrinter.HEADER_BEGIN);
        printer.print(getName().getName());
        printer.mark(this, ElementPrinter.HEADER_END); 
        printer.mark(this, ElementPrinter.ELEMENT_END);
    }
    
    //====================== SeqElement.Impl ==============================//
    
    public interface Impl extends Element.Impl  {
        
        static final long  serialVersionUID = 200401311143100L;   

        /** Get the name of this element.
         * @return the name
         */
        public Identifier getName();

        /** Set the name of this element.
         * @param name the name
         * @throws SrcException if impossible
         */
        public void setName(Identifier name) throws SrcException;
    }


    /** Default implementation of the Impl interface.
    * It just holds the property values.
    */
    static public class Memory extends Element.Memory implements SeqElement.Impl {

        static final long serialVersionUID =  200401311143201L;   

        /** Name of this element */
        private Identifier name;
        
        /** Default */
        public Memory () {
        }
        
        /** Public with Name */
        public Memory (Identifier name) {
            this.name = name;
        }
        
        /** Copy */
        public Memory (SeqElement el) {
            name = el.getName ();
        }

        /** Getter for name.
        * @return the name
        */
        public synchronized Identifier getName() {
            if (name == null) {
                name = Identifier.create(""); // NOI18N
            }
            return name;
        }

        /** Setter for name.
        * @param name the name
        */
        public synchronized void setName(Identifier name) {
            Identifier old = this.name;
            this.name = name;
            firePropertyChange (PROP_NAME, old, name);
        }
    
        /** Mark the current element in the context of this element.
        * The current element means the position for inserting new elements.
         */
        public void markCurrent(boolean mark) {
            // Do nothing, this is a terminal element.
        }
        
         /** Test if the element is marked.
        */
        public boolean isMarked () {
            return false;
        }
        
        // ================ serialization ======================================
        public Object readResolve() {
            return new SeqElement(this,null);
        }
    }
}
