/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.source;

/** General exception for the source elements hierarchy. 
 */
public class SrcException extends Exception {
    
    static final long serialVersionUID =200311101017000L;
    
    /** Create an exception. */
    public SrcException() {
        this(null);
    }
    /** Create an exception with a detail message.
    * @param msg the message
    */
    public SrcException(String msg) {
        super(msg);
    }
    
    public static class IO extends SrcException {
        private java.io.IOException nestedException;
        
        public IO(java.io.IOException cause) {
            this.nestedException = cause;
        }
        
        public IO(String msg) {
            super(msg);
        }
        
        /**
         * Returns the original I/O exception that caused the operation to fail.
         * The function may return null in a generic case.
         */
        public java.io.IOException getReason() {
            return nestedException;
        }
    }
    
    public static class Protection extends SrcException {
        private Element protectionTarget;
        
        public Protection(Element prot) {
            this.protectionTarget = prot;
        }
        
        public Element getTarget() {
            return this.protectionTarget;
        }
    }
    
    /** This is a wrapper over an unspecified VetoException thrown from a VetoableListener
     * during some change.
     */
    public static class Veto extends SrcException {
        private java.beans.PropertyVetoException reason;
        
        public Veto(java.beans.PropertyVetoException reason) {
            this.reason = reason;
        }
        
        public java.beans.PropertyVetoException getReason() {
            return this.reason;
        }
    }
}
