/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.srcgen;

import java.io.IOException;
import org.openide.text.CloneableEditorSupport;
import org.openide.text.PositionBounds;
import org.openide.text.PositionRef;
import org.starbeans.modules.seqformat.model.Binding;
import org.starbeans.modules.seqformat.model.BindingFactory;
import org.starbeans.modules.seqformat.source.SeqElement;
import org.starbeans.modules.seqformat.source.SrcElement;
import org.starbeans.modules.seqformat.source.SrcException;

/**
 * Definition of source generation package. The package exports those functions 
 * of the BindingFactory and several management functions.
 *
 * @author  Svatopluk Dedic, George Waldon
 */
public interface DocBinding extends BindingFactory {
    
    /**
     * Finds and returns the CloneableEditorSupport. This method simply delegates
     * to the environment's findEditorSupport() method.
     * @return CloneableEditorSupport instance used for binding of the model.
     */
    public CloneableEditorSupport getEditorSupport();
    
    /**
     * Causes the operation to run atomically on the document. The document
     * will be locked out for writing for the length of the operation.
     * @throw SourceException if the document cannot be loaded, locked or if some
     * of the Runnable's operation fails.
     */
    public void runAtomic(Runnable r) throws SrcException;
    
    /**
     * Causes the operation to run atomically on the document at the user-level. The document
     * will be locked out for writing for the length of the operation.
     * @throw SourceException if the document cannot be loaded, locked or if some
     * of the Runnable's operation fails.
     */
    public void enableAtomicAsUser(boolean enable);

    /**
     * Atomically updates bindings in the source, possibly inserting new ones. The 
     * hierarchy of Bindings is locked for the length of the operation. This will lock out
     * operations like get
     */
    public void updateBindings(Runnable r);
    
    /**
     * Enables or disables the code generator. If the calling code disables the generation,
     * it is <B>responsible</B> to reenable it again at the appropriate time. For example,
     * parsing updater should disable code generation before it starts to update the
     * source text model.
     */
    public void enableGenerator(boolean enableFlag);

    /**
     * Environment for the document generation package.
     */
    public static interface Env {
        /**
         * Finds CloneableEditorSupport for the code generator. The support is used
         * to load/access the document.
         */
        public CloneableEditorSupport findEditorSupport();

        /** Attempts to find "free" position in the source text, between (inclusive)
         * the given bounds. This is deferred to the user of the code generation suite.
         */
        public PositionRef  findFreePosition(PositionBounds bounds);

        /**
         * @throws IOException
         */
        public void takeLock() throws IOException;
        
        /** Create the source binding and return it.
         */
        public Binding.Source createSrcBinding(SrcElement impl, SrcText txt);
        
        /** Create the binding for a SeqElement and return it. 
         */
        public Binding.Seq createSeqBinding(SeqElement impl, SrcText txt);

    }
}

