/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 2004-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.starbeans.modules.seqformat.srcgen;

import java.util.Collection;

import org.openide.text.PositionBounds;
import org.starbeans.modules.seqformat.source.Element;
import org.starbeans.modules.seqformat.model.Binding;

/**
 * Interface for binding Elements.
 *
 * @author  sdedic, George Waldon
 * @version 
 */
public interface TextBinding extends Binding {
    
    /** Bounds for the whole element.
     */
    public static final int BOUNDS_ALL = 0;
    
    /** Bounds for header.
     */
    public static final int BOUNDS_HEADER = 1;
    
    /** Bounds for the body. 
     */
    public static final int BOUNDS_BODY = 2;


    /** Other kind of binding.
     */
     public static final int BOUNDS_OTHER = 3;

    /**
     * Returns the associated Element.
     */
    public Element getElement();
    
    /**
     * Returns the range occupied by the element.
     * @param if true, the returned range will not include the header.
     */
    public PositionBounds getElementRange(boolean defRange);

    /**
     * Returns the range occupied by the header.
     */
    public PositionBounds getHeaderRange ();
    
    /**
     * Returns a container of the specified type, or null if there is not any.
     */
    public Container getContainer(String containerType);
    
    /**
     * Updates the specified bounds. Exact types can have element-specific meaning.
     */
    public void updateBounds(int boundsType, PositionBounds bounds);
    
    /** Extension for container bindings.
     */
    public interface Container extends Binding.Container {

        public void updateChildren(Collection c);
        
        public boolean isEmpty();
    }
}
