/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.starbeans.modules.bistro.onlineservices.ncbi;

import java.net.MalformedURLException;
import java.util.Map;
import org.starbeans.modules.bistro.data.DataThingAdapter;
import org.starbeans.modules.bistro.io.TransmitterException;
import org.starbeans.modules.bistro.task.TaskExecutionException;

/**
 * This class is designed to fetch locuslink data from the NCBI database as XML.
 * 
 * @author mfortner
 */
class LocusLinkWorker extends AbstractEFetchWorker {

	public LocusLinkWorker() {
		this.db = "locuslink";
		this.rettype = "full";
		this.retmode = "xml";
	}

	public Map execute(Map inputMap) throws TaskExecutionException {
		DataThingAdapter inAdapter = new DataThingAdapter(inputMap);
		this.id = inAdapter.getString("id");

		transmitterMap.put("id", this.id);
		transmitterMap.put("db", this.db);
		transmitterMap.put("rettype", this.rettype);
		transmitterMap.put("retmode", this.retmode);
		Map results;
		try {
			results = transmit(transmitterMap);
		} catch (MalformedURLException e) {
			throw new TaskExecutionException(e);
		} catch (TransmitterException e) {
			throw new TaskExecutionException(e);
		}
		return results;
	}

	public String[] inputNames() {
		return new String[] { "id", "rettype", "retmode" };
	}

	public String[] inputTypes() {
		return new String[] { "'text/plain'", "'text/plain'", "'text/plain'" };
	}
}
