/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.starbeans.modules.bistro.onlineservices.ncbi;

import java.net.MalformedURLException;
import java.util.Map;
import java.util.logging.Logger;
import org.starbeans.modules.bistro.data.DataThingAdapter;
import org.starbeans.modules.bistro.io.TransmitterException;
import org.starbeans.modules.bistro.task.TaskExecutionException;


/**
 * This processor returns a INSD formatted nucleotide record
 * 
 * @author mfortner
 * 
 * @tavinput id The nucleotide accession.
 * @tavoutput outputText a INSD formatted nucleotide record
 */
class NucleotideINSDSeqXMLWorker extends AbstractEFetchWorker {
	
	private static Logger logger = Logger.getLogger(NucleotideINSDSeqXMLWorker.class.getName());

	public NucleotideINSDSeqXMLWorker() {
		this.db = "nucleotide";
		this.retmode = "xml";
		this.rettype = "gbc";
	}

	public Map execute(Map inputMap) throws TaskExecutionException {
		DataThingAdapter inAdapter = new DataThingAdapter(inputMap);
		this.id = inAdapter.getString("id");

		transmitterMap.put("id", this.id);
		transmitterMap.put("db", this.db);
		transmitterMap.put("rettype", this.rettype);
		transmitterMap.put("retmode", this.retmode);

		try {
			outputMap = this.transmit(transmitterMap);
		} catch (MalformedURLException e) {
			logger.severe(this.url);
			throw new TaskExecutionException(e);
		} catch (TransmitterException e) {
			throw new TaskExecutionException(e);
		}

		return outputMap;
	}

	public String[] inputNames() {
		return new String[] { "id" };
	}

	public String[] inputTypes() {
		return new String[] { "'text/plain'" };
	}

}
