/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.starbeans.modules.bistro.onlineservices.ncbi;

import java.net.MalformedURLException;
import java.util.Map;
import java.util.logging.Logger;
import org.starbeans.modules.bistro.data.DataThingAdapter;
import org.starbeans.modules.bistro.io.TransmitterException;
import org.starbeans.modules.bistro.task.TaskExecutionException;

/**
 * This class fetches a GenPept view of a protein sequence.
 * 
 * @author mfortner
 * @author George Waldon
 * 
 * @tavinput id The protein accession.
 * @tavoutput outputText a GenPept view of a protein sequence.
 */
class ProteinGenPeptWorker extends AbstractEFetchWorker {
	
	private static Logger logger = Logger.getLogger(ProteinGenPeptWorker.class.getName());

	public ProteinGenPeptWorker() {
		this.db = "protein";
		this.retmode = "text";
		this.rettype = "gp";
	}

	/**
	 * @see net.sourceforge.taverna.scuflworkers.ncbi.AbstractNCBIWorker#execute(java.util.Map)
	 */
	public Map execute(Map inputMap) throws TaskExecutionException {
		DataThingAdapter inAdapter = new DataThingAdapter(inputMap);
		this.id = inAdapter.getString("id");

		transmitterMap.put("id", this.id);
		transmitterMap.put("db", this.db);
		transmitterMap.put("rettype", this.rettype);
		transmitterMap.put("retmode", this.retmode);

		try {
			outputMap = this.transmit(transmitterMap);
		} catch (MalformedURLException e) {
			logger.severe(this.url);
			throw new TaskExecutionException(e);
		} catch (TransmitterException e) {
			throw new TaskExecutionException(e);
		} catch (Throwable th) {
			throw new TaskExecutionException(th);
		}

		return outputMap;
	}

	public String[] inputNames() {
		return new String[] { "id" };
	}

	public String[] inputTypes() {
		return new String[] { "'text/plain'" };
	}

}
