/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.starbeans.modules.bistro.onlineservices.ncbi;

import java.net.MalformedURLException;
import java.util.Map;
import org.starbeans.modules.bistro.data.DataThingAdapter;
import org.starbeans.modules.bistro.io.TransmitterException;
import org.starbeans.modules.bistro.task.TaskExecutionException;

/**
 * This class is used to fetch pubmed articles in XML form. Use this worker only
 * if you already know the pubmed id.
 * 
 * @author Mark
 * 
 * @tavinput id A comma-delimited list of PubMed IDs.
 * @tavinput rettype Indicates how complete the returned records should be:
 *           uilist, abstract, citation, medline, full (journals only)
 * @tavinput retmode Indicates the document format in which the results should
 *           be returned. Select one of the following values: "xml", "html",
 *           "text", "asn.1"
 * 
 * @tavoutput outputText An XML representation of the PubMed article
 */
class PubMedEFetchWorker extends AbstractEFetchWorker {

	public PubMedEFetchWorker() {
		this.db = "pubmed";
		this.rettype = "full";
		this.retmode = "xml";
	}

	public Map execute(Map inputMap) throws TaskExecutionException {
		DataThingAdapter inAdapter = new DataThingAdapter(inputMap);
		this.id = inAdapter.getString("id");

		String tempRetMode = inAdapter.getString("retmode");
		this.retmode = (tempRetMode != null && !tempRetMode.equals("")) ? tempRetMode : retmode;

		String tempRetType = inAdapter.getString("rettype");
		this.rettype = (tempRetType != null && !tempRetType.equals("")) ? tempRetType : rettype;

		transmitterMap.put("id", this.id);
		transmitterMap.put("db", this.db);
		transmitterMap.put("rettype", this.rettype);
		transmitterMap.put("retmode", this.retmode);
		Map results;
		try {
			results = transmit(transmitterMap);
		} catch (MalformedURLException e) {
			throw new TaskExecutionException(e);
		} catch (TransmitterException e) {
			throw new TaskExecutionException(e);
		}
		return results;
	}

	public String[] inputNames() {
		return new String[] { "id", "rettype", "retmode" };
	}

	public String[] inputTypes() {
		return new String[] { "'text/plain'", "'text/plain'", "'text/plain'" };
	}

}
